################################################################################
# Copyright 2014 spriteCloud B.V. All rights reserved.
# Generated by LapisLazuli, version 0.0.1
# Author: "spriteCloud" <info@spritecloud.com>

require 'test/unit/assertions'

include Test::Unit::Assertions

Given(/^I navigate to the (.*) test page$/) do |page|
  config = "server.url"
  if has_env?(config)
    url = env(config)
    if url == 'local_path'
      url = 'file://' + File.expand_path(File.dirname(File.dirname(__FILE__))) + '/../server/www/'
    end
    browser.goto "#{url}#{page.downcase.gsub(" ", "_")}.html"
  else
    error(:env => config)
  end
end

Given(/^I navigate to URL "(.*)"$/) do |url|
  browser.goto url
end

Given(/I click (the|a) (first|last|random|[0-9]+[a-z]+) (.*)$/) do |arg1, index, type|
  # Convert the type text to a symbol
  type = type.downcase.gsub(" ", "_")

  pick = 0
  if ["first", "last", "random"].include?(index)
    pick = index.to_sym
  else
    pick = index.to_i - 1
  end
  # Options for find
  options = {}
  # Select the correct element
  options[type.to_sym] = {}
  # Pick the correct one
  options[:pick] = pick
  # Execute the find
  type_element = browser.find(options)
  type_element.click
end

Given(/^I create a firefox browser named "(.*?)"(?: with proxy to "(.*?)")?$/) do |name, proxy_url|
  b = nil
  if proxy_url
    log.debug("Starting with profile")
    require 'selenium-webdriver'
    profile = Selenium::WebDriver::Firefox::Profile.new
    profile.proxy = Selenium::WebDriver::Proxy.new :http => proxy_url
    b = browser.create :firefox, :profile => profile
  else
    b = browser.create :firefox
  end
  scenario.storage.set(name, b)
end

Given(/^I close the browser named "(.*?)"$/) do |name|
  if scenario.storage.has? name
    b = scenario.storage.get name
    b.close
  else
    error("No item in the storage named #{name}")
  end
end

Given(/^I close the browser$/) do
  browser.close
end

When(/^I find "(.*?)" and name it "(.*?)"$/) do |id, name|
  element = browser.find(id)
  scenario.storage.set(name, element)
end

When(/^I wait for (class )?"(.*?)" and name it "(.*?)"$/) do |type, id, name|
  type = 'id' if type.nil? || type.empty?
  type.strip!
  element = browser.wait(:element => { type.to_sym => id })
  scenario.storage.set(name, element)
end

When(/^no error should be thrown when waiting for "(.*?)"$/) do |id|
  begin
    elm = browser.wait(
      :element => { :id => id },
      :timeout => 3,
      :throw => false
    )
  rescue
    raise 'An error was thrown using a wait funtion with :throw => false'
  end
  if not elm.nil?
    raise 'No element was suppose to be found, but it did have a value after the wait action.'
  end
end

xpath_fragment = nil
Given(/^I specify a needle "(.+?)" and a node "(.+?)" (and an empty separator )?to xp_contains$/) do |needle, node, empty_sep|
  if empty_sep.nil?
    xpath_fragment = xp_contains(node, needle)
  else
    xpath_fragment = xp_contains(node, needle, '')
  end
end

Then(/^I expect an xpath fragment "(.*?)"$/) do |fragment|
  assert fragment == xpath_fragment, "Fragment was not as expected: got '#{xpath_fragment}' vs expected '#{fragment}'."
end

Then(/^I expect the fragment "(.*?)" to find (\d+) element\(s\)\.$/) do |fragment, n|
  elems = browser.elements(:xpath => "//div[#{fragment}]")
  assert n.to_i == elems.length, "Mismatched amount: got #{elems.length} vs. expected #{n}"
end

elems = []
Given(/^I search for elements where node "(.+?)" contains "(.+?)" and not "(.+?)"$/) do |node, first, second|
  clause = xp_and(xp_contains(node, first), xp_not(xp_contains(node, second)))
  elems = browser.elements(:xpath => "//div[#{clause}]")
end

Then(/^I expect to find (\d+) elements\.$/) do |n|
  assert n.to_i == elems.length, "Mismatched amount: got #{elems.length} vs. expected #{n}"
end

When(/^I go to "(.*?)"$/) do |url|
  browser.goto url
end

Then(/^I should be able to click the first button by event$/) do
  elem = browser.button(:id => 'first')
  browser.on_click(elem)
  browser.wait(
    :timeout => 1,
    :text => 'first clicked',
    :groups => ['wait'],
  )
end

Then(/^I should be able to click the first button by using JavaScript$/) do
  elem = browser.button(:id => 'first')
  browser.js_click(elem)
  browser.wait(
    :timeout => 1,
    :text => 'first clicked',
    :groups => ['wait'],
  )
end

Then(/^I should be able to click the first button by click type (.*?)$/) do |type|
  elem = browser.button(:id => 'first')
  browser.click_type(elem, type)
  browser.wait(
    :timeout => 1,
    :text => 'first clicked',
    :groups => ['wait'],
  )
end

Then(/^I should be able to click the first button by force click$/) do
  elem = browser.button(:id => 'first')
  browser.force_click(elem)
  browser.wait(
    :timeout => 1,
    :text => 'first clicked',
    :groups => ['wait'],
  )
end

Given(/^I set environment variable "(.*?)" to "(.*?)"$/) do |var, val|
  ENV[var] = val
end

Given(/^I annotate a step with (.*?)$/) do |data|
  annotate data
end

Then(/^the report should include (.*?) and (.*?) in the correct place$/) do |data1, data2|
  # Our test completely ignores the "scope" part, because the scope is the
  # scenario/example anyway. Instead we just check the stored values are
  # as expected.
  annotations.each do |scope, values|
    assert ([[data1], [data2]] == values), "Stored values: #{values}, expected [[#{data1}], [#{data2}]]"
  end
end

Given(/^I use browser bindings "(.*?)"$/) do |bindings|
  require 'selenium-webdriver'
  case bindings
  when '1'
    profile = Selenium::WebDriver::Firefox::Profile.new
    profile['general.useragent.override'] = "CUSTOM-USER-AGENT"
    browser.restart :firefox, options: { profile: profile }
  when '2'
    browser.restart :chrome, options: { args: ['--user-agent=CUSTOM-CHROME-USER-AGENT'] }
  when '3'
    browser.restart :chrome, options: { args: ['--start-maximized'] }
  else
    error "Requested binding setup does not exist. Requested #{bindings}"
  end
end

Given(/^I restart the browser to device setting "(.*?)"$/) do |device|
  browser.restart :chrome, device: device
end

When(/^I take a screenshot$/) do
  browser.take_screenshot
end

When(/^I fail this step$/) do
  error 'This step fails because it is suppose to'
end